import importlib
import logging
import pkgutil


logger = logging.getLogger(__name__)


def extend_superapp_settings(main_settings, superapp_apps):
    for importer, modname, ispkg in pkgutil.iter_modules(superapp_apps.__path__):
        submodule_name = f"{superapp_apps.__name__}.{modname}.settings"

        try:
            settings_module = importlib.import_module(submodule_name)
        except ModuleNotFoundError as e:
            if f"No module named '{submodule_name}'" in str(e):
                continue
            raise e

        if hasattr(settings_module, "extend_superapp_settings"):
            settings_module.extend_superapp_settings(main_settings)



import os
import subprocess
import shutil

import sysrsync

import click


def clone_repo(repo_url, target_directory):
    click.echo("Forking the repository...")
    if not is_gh_installed():
        raise click.ClickException(
            "gh is not installed. Please install it before running this command using `brew install gh`."
        )
    result = subprocess.run(["gh", "repo", "fork", repo_url, "--clone", target_directory], capture_output=True, text=True)
    if result.returncode != 0:
        raise click.ClickException(f"Failed to fork repository. Error: {result.stderr}")


def create_pr(fork_directory):
    click.echo("Creating the PR...")
    if not is_gh_installed():
        raise click.ClickException(
            "gh is not installed. Please install it before running this command using `brew install gh`."
        )
    result = subprocess.run(["gh", "pr", "create", "--web", "--fill"], capture_output=True, text=True, cwd=fork_directory)
    if result.returncode != 0:
        raise click.ClickException(f"Failed to create PR. Error: {result.stderr}")


def sync_directories(src, dst, copier_configuration=None):
    """
    Syncs files from src to dst, ignoring files specified in the gitignore file
    and additional user-provided regex patterns. Removes files from dst that
    don't exist in src respecting gitignore_file and ignore_patterns.

    :param src: Source directory path
    :param dst: Destination directory path
    :param copier_configuration: Copier configuration
    """
    if not is_rsync_installed():
        raise click.ClickException(
            "rsync is not installed. Please install it before running this command."
        )

    options = [
        '-av',
        '--delete',
    ]
    copier_excludes = (
        copier_configuration.get('_exclude', []) if copier_configuration else []
    ) + (
        copier_configuration.get('_rsync_exclude', []) if copier_configuration else []
    )
    copied_excluded_files = [
        c
        for c in copier_excludes
        if not c.startswith('!')
    ]
    copied_included_files = [
        c
        for c in copier_excludes
        if c.startswith('!')
    ]
    for c in copied_included_files:
        options.append(f"--include={c[1:]}")
    sysrsync.run(
        source=src,
        destination=dst,
        options=options,
        exclusions=[
                            ".DS_Store",
                            "venv",
                            "superapp_fork",
                            ".git",
                            ".copier-answers.yml",
                            "{{_copier_conf.answers_file}}.jinja",
                        ] + copied_excluded_files,
        verbose=True,
    )


def remove_directory_if_exists(directory_path):
    if os.path.exists(directory_path) and os.path.isdir(directory_path):
        shutil.rmtree(directory_path)



def is_gh_installed():
    try:
        result = subprocess.run(["gh", "--version"], capture_output=True, text=True)
        return result.returncode == 0
    except FileNotFoundError:
        return False


def is_git_installed():
    try:
        result = subprocess.run(["git", "--version"], capture_output=True, text=True)
        return result.returncode == 0
    except FileNotFoundError:
        return False


def is_github_installed():
    try:
        result = subprocess.run(["github", "help"], capture_output=True, text=True)
        return result.returncode == 0
    except FileNotFoundError:
        return False


def is_rsync_installed():
    try:
        result = subprocess.run(["rsync", "--version"], capture_output=True, text=True)
        return result.returncode == 0
    except FileNotFoundError:
        return False


def display_git_diff():
    if not is_git_installed():
        raise click.ClickException("git is not installed. Please install it before running this command.")

    try:
        result = subprocess.run(["git", "--no-pager", "diff"], capture_output=True, text=True)
        return result.returncode == 0
    except FileNotFoundError:
        return False


def open_repo_in_github(repo_directory):
    click.echo(f"Please review and commit the changes in the forked repository {repo_directory}")
    if is_github_installed():
        click.echo("Opening the repository in GitHub Desktop...")
        try:
            result = subprocess.run(["github", "open", repo_directory], capture_output=True, text=True)
            return result.returncode == 0
        except FileNotFoundError:
            return False



import os
import re
from pathlib import Path

import click
import yaml
from copier import run_copy, run_update, Worker

from .utils import sync_directories, clone_repo, open_repo_in_github, create_pr, remove_directory_if_exists
import tempfile

def validate_github_repo(ctx, param, value):
    # Regular expressions for GitHub repository URLs
    github_https_regex = r'^https:\/\/github\.com\/[a-zA-Z0-9\-_]+\/[a-zA-Z0-9\-_]+$'
    github_ssh_regex = r'^git@github\.com:[a-zA-Z0-9\-_]+\/[a-zA-Z0-9\-_]+\.git$'

    # Check if the value matches either regex
    if not re.match(github_https_regex, value) and not re.match(github_ssh_regex, value):
        raise click.BadParameter('The repository URL must be a valid GitHub HTTPS or SSH URL.')

    return value


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    '--template-repo',
    default='https://github.com/django-superapp/django-superapp-default-project',
    callback=validate_github_repo,
    help='SuperApp GitHub repository URL.'
)
@click.argument('target_directory')
def bootstrap_project(template_repo, target_directory):
    """Bootstrap a project into target directory."""

    run_copy(
        template_repo,
        str(target_directory),
        data={'project_name': os.path.basename(target_directory)},
    )


@cli.command()
@click.option(
    '--template-repo',
    callback=validate_github_repo,
    help='SuperApp GitHub repository URL.'
)
@click.argument('target_directory')
def bootstrap_app(template_repo, target_directory):
    """Create an app inside the project."""
    current_directory = os.getcwd()
    if not current_directory.endswith("superapp/apps"):
        raise click.ClickException(
            "You must run this command inside the 'superapp/apps' directory."
        )

    run_copy(
        template_repo,
        str(target_directory),
        data={'app_name': os.path.basename(target_directory)},
    )


@cli.command()
@click.argument('target_directory')
def pull_template(target_directory):
    """Update the local template with the remote changes."""

    run_update(
        str(target_directory),
        overwrite=True,
    )


@cli.command()
@click.argument('target_directory')
def push_template(target_directory):
    """Push the local changes to the remote template."""
    fork_directory = tempfile.mkdtemp()
    try:
        with Worker(dst_path=Path(target_directory)) as worker:
            repo_url = worker.subproject.last_answers.get('_src_path')

        try:
            with open(f"{target_directory}/copier.yml", 'r') as file:
                copier_configuration = yaml.safe_load(file)
        except OSError:
            copier_configuration = {}

        if not repo_url:
            raise click.ClickException(
                "The target directory does not have a remote repository configured."
            )

        click.echo(f"Forking the repository in {fork_directory} directory...")
        clone_repo(
            repo_url=repo_url,
            target_directory=fork_directory,
        )

        click.echo("Syncing the directories...")
        sync_directories(
            src=target_directory,
            dst=fork_directory,
            copier_configuration=copier_configuration,
        )

        open_repo_in_github(fork_directory)
        click.secho("Do not commit sensitive information to the remote repository!", fg='yellow', bold=True)
        if not click.confirm('Have you pushed your commits to the remote repository?'):
            raise click.Abort()

        create_pr(fork_directory)
    except Exception as e:
        raise e
    finally:
        click.echo("Cleaning up...")
        remove_directory_if_exists(fork_directory)


if __name__ == '__main__':
    cli()
